import { useState, useEffect, useRef } from "react";
import { VideoJsPlayer } from "video.js";

import { ITraining } from "../../training/types";
import { getExerciseList, getTrainingTime, getPosition } from "../utils";

import Countdown from "./Countdown";
import VideoPlayer from "./VideoPlayer";
import AudioPlayer from "./AudioPlayer";
import { IExerciseItem } from "../types";
import { Howl } from "howler";
import { useTimer } from "../hooks";

const Timer = ({ training }: { training: ITraining }) => {
  const [time, timer] = useTimer({ tickPeriod: 100 });

  const [state, setState] = useState({
    exerciseList: [] as IExerciseItem[],
    totalTime: 0
  });

  const rosie = useRef(
    new Howl({
      src: ["/media/ROSIE.mp3"],
      sprite: {
        ttt: [114, 2956 - 114],
        five: [10211, 10883 - 10211],
        four: [11292, 11873 - 11292],
        three: [12251, 12809 - 12251],
        two: [13208, 13930 - 13208],
        one: [14376, 14948 - 14376],
        go: [18219, 18735 - 18219],
        rest: [18988, 19615 - 18988]
      }
    })
  );

  useEffect(() => {
    //console.log("effect 1");
    rosie.current.play("ttt");
    const exerciseList = getExerciseList(training.blocks);
    const totalTime = getTrainingTime(exerciseList);
    setState({ ...state, exerciseList, totalTime });
  }, [training]);

  //console.log("is it over?", time, state.totalTime, trainingOver);

  const {
    currentExercise,
    previousExercise,
    nextExercise,
    exerciseTime
  } = getPosition(state.exerciseList, timer.time);
  //console.log("aaa", time, currentExercise);

  useEffect(() => {
    const countdown = currentExercise
      ? currentExercise.duration + currentExercise.offset - timer.intTime + 1
      : 0;
    if (timer.running && rosie.current) {
      if (countdown === 5) rosie.current.play("five");
      else if (countdown === 4) rosie.current.play("four");
      else if (countdown === 3) rosie.current.play("three");
      else if (countdown === 2) rosie.current.play("two");
      else if (countdown === 1) rosie.current.play("one");
      if (exerciseTime < 1) {
        if (currentExercise && currentExercise.exercise === "Rest")
          rosie.current.play("rest");
        else rosie.current.play("go");
      }
    }
  }, [time, timer.running]);

  const video: { current: VideoJsPlayer | undefined } = useRef();
  //const audio: { current: Howl | undefined } = useRef();

  function startTimer() {
    if (time >= state.totalTime) return;
    timer.start();
    if (video.current) video.current.play();
    //if (audio.current) audio.current.play();
    //console.log("Timer started.");
  }

  function stopTimer() {
    timer.stop();
    if (video.current) video.current.pause();
    //if (audio.current) audio.current.pause();
    //console.log("stopped");
  }

  function forward() {
    if (nextExercise) {
      timer.setTime(nextExercise.offset);
    }
  }

  function back() {
    if (previousExercise) {
      timer.setTime(previousExercise.offset);
    }
  }

  //console.log("current state:", currentExercise, state);

  return (
    <>
      <div>
        {/*<label htmlFor="rest">Rest</label>
        <input type="number" min="25" max="60" step="5" defaultValue="25" />*/}
        <button onClick={back}>back</button>
        <button onClick={timer.running ? stopTimer : startTimer}>
          {timer.running ? "stop" : "start"}
        </button>
        <button onClick={forward}>forward</button>
      </div>
      <div className="content">
        <Countdown
          seconds={
            currentExercise ? currentExercise.duration - exerciseTime : 0
          }
          totalPercent={timer.time / state.totalTime}
          exercisePercent={
            exerciseTime / (currentExercise ? currentExercise.duration : 1)
          }
          onClick={timer.running ? stopTimer : startTimer}
        />
        <div className="header">current exercise</div>
        <div className="exercise">
          {currentExercise ? currentExercise.exercise : "😎"}
        </div>
        <div className="header">next up</div>
        <div className="exercise">
          {nextExercise ? nextExercise.exercise : "😎"}
        </div>
        <VideoPlayer
          src={
            currentExercise && currentExercise.video
              ? currentExercise.video
              : ""
          }
          getVideoHandle={(videoHandle: VideoJsPlayer) =>
            (video.current = videoHandle)
          }
        />
      </div>

      <style jsx>{`
        .content {
          text-align: center;
        }

        .header {
          margin-top: 0.4em;
          font-size: 90%;
        }

        .exercise {
          font-size: 110%;
          font-weight: 900;
        }

        :global(.content .video-js) {
          margin: 20px auto;
        }

        @media (min-width: 700px) {
          .content {
            text-align: center;
            display: grid;
            grid-auto-flow: dense;
            grid-template-columns: 1fr auto;
          }
          .header,
          .exercise {
            grid-column: 1/2;
          }
        }
      `}</style>
    </>
  );
};

export default Timer;
